import React from 'react';
import StatusChip from './StatusChip';
import { Skeleton, Tooltip } from '@mui/material';
import {
  checkStatus,
  compareDRepIDs,
  convertHexToCIP129,
  formattedAda,
} from '@/lib';
import MetadataViewer from './MetadataViewer';
import DRepSocialLinks from './DRepSocialLinks';
import DRepAvatarCard from './DRepAvatarCard';
import { useCardano } from '@/context/walletContext';
import { useDRepContext } from '@/context/drepContext';
import { useGetDRepMetadataQuery } from '@/hooks/useGetDRepMetadataQuery';
import DRepIdHolder from './DRepIdHolder';
import { useDelegateTodRep } from '@/hooks/useDelegateToDRep';
import ClaimProfileButton from './ClaimProfileButton';
import Button from './Button';
import { useGetAdaHolderCurrentDelegationQuery } from '@/hooks/useGetAdaHolderCurrentDelegationQuery';
import { useGetOwnership } from '@/hooks/useGetOwnership';

type DrepClaimProfileCardProps = {
  drep: any;
  voterId: string;
  loading: boolean;
};

const DrepClaimProfileCard = ({
  drep,
  voterId,
  loading,
}: DrepClaimProfileCardProps) => {
  const { stakeKey, dRepIDBech32 } = useCardano();
  const { isLoggedIn } = useDRepContext();
  const { delegate, isDelegating } = useDelegateTodRep();
  const { metadata, isMetadataLoading, metadataError } =
    useGetDRepMetadataQuery(voterId);
  const { ownership } = useGetOwnership({
    drepId: drep?.view,
    voterId: dRepIDBech32,
  });
  const { currentDelegation } = useGetAdaHolderCurrentDelegationQuery(stakeKey);
  const isDelegated = compareDRepIDs(drep?.view, currentDelegation?.drep_view);

  return (
    <div className="flex flex-col gap-5 bg-white bg-opacity-50 px-5 py-10">
      <DRepAvatarCard
        loading={loading}
        imageSrc={metadata?.body?.image?.contentUrl}
      />
      {drep?.type !== 'scripted' && drep?.type !== 'voting_option' && (
        <ClaimProfileButton
          label="Claim this profile"
          drepToBeClaimed={drep?.view}
        />
      )}
      {!isDelegated && (
        <Button
          className="w-full"
          disabled={!!isDelegating}
          handleClick={() => {
            delegate(drep?.view, { isRetired: drep?.retired });
          }}
        >
          Delegate
        </Button>
      )}
      <div className="flex flex-row gap-2">
        {drep?.type === 'scripted' && <StatusChip status="Scripted" />}
        {drep?.type === 'voting_option' && (
          <StatusChip status="Voting Option" />
        )}
        {drep?.type !== 'scripted' && drep?.type !== 'voting_option' && (
          <StatusChip status="Not claimed" />
        )}
        {drep?.retired && drep?.type !== 'voting_option' && (
          <StatusChip status="Retired" />
        )}
        <StatusChip
          status={
            drep?.type === 'voting_option'
              ? 'Active'
              : checkStatus(drep?.active)
          }
        />
        {isDelegated && (
          <Tooltip title="You have delegated to this DRep">
            <span>
            <button>
              <StatusChip status="Your DRep" />
            </button>
            </span>
          </Tooltip>
        )}
      </div>
      <div className="flex items-center gap-4">
        <div>
          <p className="font-bold">Voting power</p>
          <p className="flex items-center gap-3 font-normal">
            {loading ? (
              <Skeleton animation={'wave'} width={50} height={20} />
            ) : drep?.voting_power != null ? (
              `₳ ${formattedAda(drep?.voting_power, 2)}`
            ) : (
              '-'
            )}
          </p>
        </div>
        <div>
          <p className="font-bold">Live Stake</p>
          <p className="flex items-center gap-3 font-normal">
            {loading ? (
              <Skeleton animation={'wave'} width={50} height={20} />
            ) : drep?.live_stake != null ? (
              `₳ ${formattedAda(drep?.live_stake, 2)}`
            ) : (
              '-'
            )}
          </p>
        </div>
      </div>
      <div>
        <p className="font-bold">Total delegation</p>
        <p>
          {loading ? (
            <Skeleton animation={'wave'} width={150} height={20} />
          ) : (
            `${drep?.delegation_vote_count || 0} ${drep?.delegation_vote_count > 1 ? 'Delegators' : 'Delegator'}`
          )}
        </p>
      </div>
      <fieldset className="rounded-2xl border border-blue-100 px-2">
        <legend className="font-bold">DRep ID</legend>
        <div className="flex flex-col items-start justify-center gap-1 divide-y divide-blue-100 pb-2">
          <DRepIdHolder
            loading={loading}
            drepId={convertHexToCIP129(drep?.has_script, drep?.chain_id)}
            isCIP129={true}
          />
          <DRepIdHolder
            loading={loading}
            drepId={drep?.view}
            isCIP129={false}
            className="pt-1"
          />
        </div>
      </fieldset>
      <DRepSocialLinks links={metadata?.body?.references} />
      <div>
        {isMetadataLoading ? (
          <Skeleton animation={'wave'} width={150} height={20} />
        ) : (
          <MetadataViewer
            metadata={metadata}
            isMetadataLoading={isMetadataLoading}
            metadataError={metadataError}
            metadataUrl={drep?.metadata_url}
          />
        )}
      </div>
      {ownership && ownership.result === true && isLoggedIn && (
        <div className="flex max-w-prose flex-col gap-2">
          <ClaimProfileButton
            label="Claim your profile to update"
            drepToBeClaimed={drep?.view}
          />
        </div>
      )}
    </div>
  );
};

export default DrepClaimProfileCard;
